/*  $Id: eeeapplet.c 2257 2006-12-19 19:49:00Z nick $
 *
 *  Copyright (c) 2006-2007 John Doo <john@foo.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include <gtk/gtk.h>
#include <libxfce4util/libxfce4util.h>
#include <libxfce4panel/xfce-panel-plugin.h>
#include <libxfce4panel/xfce-hvbox.h>
#include <libnotify/notify.h>
#include <stdio.h>
#include <sys/types.h>
#include <dirent.h>
#include <math.h>


#include "eeeapplet.h"
#include "eeeapplet-dialogs.h"

/* default settings */
#define DEFAULT_SETTING1 NULL
#define DEFAULT_SETTING2 1
#define DEFAULT_SETTING3 FALSE



/* prototypes */
static void
eeeapplet_construct (XfcePanelPlugin *plugin);


/* register the plugin */
XFCE_PANEL_PLUGIN_REGISTER_EXTERNAL (eeeapplet_construct);


int isDir(const char path[]) {
      DIR * dir_pointer;
       dir_pointer=opendir(path);

       if (!dir_pointer) {
                closedir(dir_pointer);
                return 0;
       } else {
                closedir(dir_pointer);
                return 1;
        }
}

void
update_battery(EEEAppletPlugin *eeeapplet) {
  FILE *fp;
  char str[128];
  char cache[20];
  int x;int y;
  int curcharge=0;
  char curbatterystate[20];
  int first=batterymax;

  if (batterymax==-1) {
	  if((fp = fopen(BATTERYINFO, "r"))!=NULL) {

		while(!feof(fp)) {
			if(fgets(str, 126, fp))  {
				if (strncmp(str,"last full capacity:",19)==0) {
					y=0;
					for (x=19;x<strlen(str)-1;x++)
						if ((str[x]>47)&&(str[x]<58)) { cache[y]=str[x]; y++;}
					cache[y]=0;
					break;
				}
			}
		}
		fclose(fp);
		batterymax=atoi(cache);
	  }
  }
  if((fp = fopen(BATTERYSTATE, "r"))!=NULL) {
	while(!feof(fp)) {
		if(fgets(str, 126, fp))  {
			if (strncmp(str,"charging state:",15)==0) {
				x=16;y=0;
				while (str[x]==32) x++;
				for (;x<strlen(str)-1;x++) { curbatterystate[y]=str[x];y++; }
				curbatterystate[y]=0;
			}
			if (strncmp(str,"remaining capacity:",19)==0) {
				y=0;
				for (x=19;x<strlen(str)-1;x++)
					if ((str[x]>47)&&(str[x]<58)) { cache[y]=str[x]; y++;}
				cache[y]=0;
				curcharge=atoi(cache);
			}
		}
	}
	fclose(fp);
	
	if (curcharge>batterymax) curcharge=batterymax;
	gtk_progress_bar_update(eeeapplet->gauge,(gdouble)curcharge/batterymax);
	
	if (!batterylow&&((((gdouble)curcharge/batterymax)*100)<BATTERYLOWALERT)) {
		NotifyNotification *n; 
		n = notify_notification_new (_("Battery low"), 
									 NULL,
									 NULL, NULL);
		notify_notification_set_urgency (n, NOTIFY_URGENCY_CRITICAL);
		notify_notification_set_timeout (n, 5000); // 5 seconds
		notify_notification_attach_to_widget(n,eeeapplet->gauge);

		if (!notify_notification_show (n, NULL)) 
		{
		fprintf(stderr, "failed to send notification\n");
		return 1;
		}
		g_object_unref(G_OBJECT(n));
		batterylow=1;
	} else if (strcmp(curbatterystate,batterystate)!=0) {
		if (first!=-1) {
			NotifyNotification *n; 
			gchar *message;
			message=g_strdup_printf(_("Battery is now %s."),curbatterystate);
			n = notify_notification_new (_("Battery"), 
										 message,
										 NULL, NULL);
			notify_notification_set_urgency (n, NOTIFY_URGENCY_NORMAL);
			notify_notification_set_timeout (n, 5000); // 5 seconds
			notify_notification_attach_to_widget(n,eeeapplet->gauge);

			if (!notify_notification_show (n, NULL)) 
			{
			fprintf(stderr, "failed to send notification\n");
			return 1;
			}
			g_object_unref(G_OBJECT(n));
			batterylow=0;
			g_free(message);
		}
		strcpy(batterystate,curbatterystate);
	}
  }
}

void
update_cpu(EEEAppletPlugin *eeeapplet) {
  FILE *fp;
  char str[128];
  double x;
  char unit[4];
  int dec;int pint;

  if((fp = fopen(CPUSPEED, "r"))!=NULL) {
	while(!feof(fp)) {
		if(fgets(str, 126, fp))  {
			x=atoi(str);
			if (x>1000) { x=x/1000; strcpy(unit,"Mhz"); }
			if (x>1000) { x=x/1000; strcpy(unit,"Ghz"); }
			pint=floor(x);
			dec=floor((x-pint)*100);
			if (dec>0) 
					sprintf(str,"%d,%d%s",pint,dec,unit);
			else
					sprintf(str,"%d%s",pint,unit);			
			gtk_label_set_text(eeeapplet->label,str);
			break;
		}
	}
	fclose(fp);
  }
}

void
eeeapplet_save (XfcePanelPlugin *plugin,
             EEEAppletPlugin    *eeeapplet)
{
  /* cancelled */
  return;

  XfceRc *rc;
  gchar  *file;

  /* get the config file location */
  file = xfce_panel_plugin_save_location (plugin, TRUE);

  if (G_UNLIKELY (file == NULL))
    {
       DBG ("Failed to open config file");
       return;
    }

  /* open the config file, read/write */
  rc = xfce_rc_simple_open (file, FALSE);
  g_free (file);

  if (G_LIKELY (rc != NULL))
    {
      /* save the settings */
      DBG(".");
      if (eeeapplet->setting1)
        xfce_rc_write_entry    (rc, "setting1", eeeapplet->setting1);

      xfce_rc_write_int_entry  (rc, "setting2", eeeapplet->setting2);
      xfce_rc_write_bool_entry (rc, "setting3", eeeapplet->setting3);

      /* close the rc file */
      xfce_rc_close (rc);
    }
}



static void
eeeapplet_read (EEEAppletPlugin *eeeapplet)
{
  /* cancelled */
  return;
  
  XfceRc      *rc;
  gchar       *file;
  const gchar *value;

  /* get the plugin config file location */
  file = xfce_panel_plugin_save_location (eeeapplet->plugin, TRUE);

  if (G_LIKELY (file != NULL))
    {
      /* open the config file, readonly */
      rc = xfce_rc_simple_open (file, TRUE);

      /* cleanup */
      g_free (file);

      if (G_LIKELY (rc != NULL))
        {
          /* read the settings */
          value = xfce_rc_read_entry (rc, "setting1", DEFAULT_SETTING1);
          eeeapplet->setting1 = g_strdup (value);

          eeeapplet->setting2 = xfce_rc_read_int_entry (rc, "setting2", DEFAULT_SETTING2);
          eeeapplet->setting3 = xfce_rc_read_bool_entry (rc, "setting3", DEFAULT_SETTING3);

          /* cleanup */
          xfce_rc_close (rc);

          /* leave the function, everything went well */
          return;
        }
    }

  /* something went wrong, apply default values */
  DBG ("Applying default settings");

  eeeapplet->setting1 = g_strdup (DEFAULT_SETTING1);
  eeeapplet->setting2 = DEFAULT_SETTING2;
  eeeapplet->setting3 = DEFAULT_SETTING3;
}

static gint
select_network (GtkWidget *widget,  NetworkData *data) {
	NotifyNotification *n; 
	gchar *title=NULL;
	gchar *body=NULL;
	gchar *icon=NULL;
	NotifyUrgency urgency=NOTIFY_URGENCY_NORMAL;


	gint status;
	gchar cmdline[300];
	gchar i[300];
	int x;
	
	sprintf(cmdline,"sudo netcfg2 -d \"%s\"",data->name);
	int ret=g_spawn_command_line_sync(cmdline,NULL,NULL,&status,NULL);
	if (status==0) { // disconnected ok, remove from list
		title=g_strdup(_("Disconnected"));
		body=g_strdup_printf(_("Disconnected from network \"%s\"."),data->name);
		icon=g_strdup(ICON_DISCONNECTED);
		for (x=0;x<MAXNETWORKS;x++) if (strcmp(connected[x],data->name)==0) { strcpy(connected[x],"");break; }
		gtk_image_set_from_icon_name(data->eeeapplet->image,ICON_DISCONNECTED,xfce_panel_plugin_get_size(data->eeeapplet));
	} else { // probably was disconnected. Try to connect	
		sprintf(cmdline,"sudo netcfg2 \"%s\"",data->name);
		int ret=g_spawn_command_line_sync(cmdline,NULL,NULL,&status,NULL);
		if (status==0) { // connected, add to connection list
			title=g_strdup("Connected");
			body=g_strdup_printf("Connected to network \"%s\".",data->name);
			icon=g_strdup(ICON_CONNECTED);
			urgency=NOTIFY_URGENCY_LOW;
			for (x=0;x<MAXNETWORKS;x++) if (strcmp(connected[x],"")==0) { strcpy(connected[x],data->name); break; }
			gtk_image_set_from_icon_name(data->eeeapplet->image,ICON_CONNECTED,xfce_panel_plugin_get_size(data->eeeapplet));
		} else { // something gone wrong...
			title=g_strdup(_("Error"));
			body=g_strdup_printf(_("Error connecting to \"%s\"."),data->name);
			icon=g_strdup(ICON_ERROR);
			urgency=NOTIFY_URGENCY_CRITICAL;
			gtk_image_set_from_icon_name(data->eeeapplet->image,ICON_ERROR,xfce_panel_plugin_get_size(data->eeeapplet));
		}
	}

	n = notify_notification_new (title, 
                                 body,
                                 icon, NULL);
    notify_notification_set_urgency (n, urgency);
    notify_notification_set_timeout (n, 5000); // 5 seconds
	notify_notification_attach_to_widget(n,data->eeeapplet->image);

    if (!notify_notification_show (n, NULL)) 
    {
	fprintf(stderr, "failed to send notification\n");
	return 1;
    }
    g_object_unref(G_OBJECT(n));
	g_free (icon);
	g_free (body);
	g_free (title);    
	return 0;

}


static gint tick(GtkWidget *widget) {
	if (timetobattery==0) {
		update_battery(widget);
		timetobattery=BATTERYUPDATETIME;
	} else timetobattery--;
	if (timetocpu==0) {
		update_cpu(widget);
		timetocpu=CPUUPDATEITIME;
	} else timetocpu--;
	return TRUE;
}

static gint
popup_scripts (GtkWidget *widget, EEEAppletPlugin    *eeeapplet)
{
	  gtk_menu_popup (eeeapplet->scriptmenu, NULL, NULL, NULL, NULL, NULL, NULL);
	  return TRUE;
}

static gint script_run (gchar
*user_data)

{
	
	if (!g_spawn_command_line_async(user_data,NULL)) { // disconnected ok, remove from list
		NotifyNotification *n; 
		n = notify_notification_new (_("Error"), 
									 _("Error during execution of the script."),
									 ICON_SCRIPT, NULL);
		notify_notification_set_urgency (n, NOTIFY_URGENCY_CRITICAL);
		notify_notification_set_timeout (n, 5000); // 5 seconds
		notify_notification_attach_to_widget(n,eeeapplet->scriptbutton);

		if (!notify_notification_show (n, NULL)) 
		{
		fprintf(stderr, "failed to send notification\n");
		return 1;
		}
		g_object_unref(G_OBJECT(n));

	}

	return TRUE;
}


static gint
popup_networks (GtkWidget *widget, EEEAppletPlugin    *eeeapplet)
{
       DIR *dp;
       struct dirent *ep;
       char fullpath[MAXNAMELENGTH];
       GtkWidget *image;
       
       
 	int i;	
 	int x;
 for (i=0;i<MAXNETWORKS;i++) {
 	gtk_widget_destroy(eeeapplet->options[i]);
 }
 
     
       dp = opendir (NETPROFILESDIR);
     
       if (dp != NULL)
         {
          i=0;
           while (ep = readdir (dp)) {
      	      sprintf(fullpath,"%s%s",NETPROFILESDIR,ep->d_name);
              if ((strncmp(ep->d_name,".",1)!=0)&&(!isDir(fullpath))) {
			  data[i].eeeapplet=eeeapplet;
			  sprintf(data[i].name,"%s",ep->d_name);
			  
			  for (x=0;x<MAXNETWORKS;x++) if (strcmp(connected[x],ep->d_name)==0) {x=-1;break;}
			  
			  image=gtk_image_new_from_icon_name((x==-1?ICON_CONNECTED:ICON_DISCONNECTED),GTK_ICON_SIZE_MENU);
			  
			  eeeapplet->options[i]=gtk_image_menu_item_new_with_label(ep->d_name); 
			  gtk_image_menu_item_set_image(eeeapplet->options[i],image);
			  
			  
			  gtk_signal_connect( GTK_OBJECT(eeeapplet->options[i]), "activate",GTK_SIGNAL_FUNC(select_network), &data[i]);
			  gtk_menu_append(eeeapplet->menu,eeeapplet->options[i]);
			  gtk_widget_show(eeeapplet->options[i]);
			  
			  i++;
			  if (i==MAXNETWORKS) break;
              }
           }
           (void) closedir (dp);
         }

	  gtk_menu_popup (eeeapplet->menu, NULL, NULL, NULL, NULL, NULL, NULL);
	  return TRUE;
}

static EEEAppletPlugin *
eeeapplet_new (XfcePanelPlugin *plugin)
{
  GtkOrientation  orientation;
  GtkWidget      *button;
  GtkWidget *menu;
  GtkWidget *tmpimage;


  /* allocate memory for the plugin structure */
  eeeapplet = panel_slice_new0 (EEEAppletPlugin);

  /* pointer to plugin */
  eeeapplet->plugin = plugin;

  /* read the user settings */
  eeeapplet_read (eeeapplet);

  /* get the current orientation */
  orientation = xfce_panel_plugin_get_orientation (plugin);

  /* create some panel widgets */
  eeeapplet->ebox = gtk_event_box_new ();
  gtk_widget_show (eeeapplet->ebox);

  eeeapplet->hvbox = xfce_hvbox_new (orientation, FALSE, 2);
  gtk_widget_show (eeeapplet->hvbox);
  gtk_container_add (GTK_CONTAINER (eeeapplet->ebox), eeeapplet->hvbox);

  /* network button */
  eeeapplet->button = xfce_create_panel_button();
  gtk_widget_show(eeeapplet->button);
  gtk_container_add(GTK_CONTAINER(eeeapplet->hvbox),eeeapplet->button);

  eeeapplet->image=gtk_image_new();
  gtk_image_set_from_icon_name(eeeapplet->image,ICON_DISCONNECTED,xfce_panel_plugin_get_size(eeeapplet));
  gtk_widget_show (eeeapplet->image);
  gtk_container_add(GTK_CONTAINER(eeeapplet->button),eeeapplet->image);
  
  eeeapplet->menu=menu=gtk_menu_new();
  gtk_menu_attach_to_widget(eeeapplet->menu,eeeapplet->button,NULL);

  /* script button */
  eeeapplet->scriptbutton = xfce_create_panel_button();
  gtk_widget_show(eeeapplet->scriptbutton);
  gtk_container_add(GTK_CONTAINER(eeeapplet->hvbox),eeeapplet->scriptbutton);

  tmpimage=gtk_image_new();
  gtk_image_set_from_icon_name(tmpimage,ICON_SCRIPT,xfce_panel_plugin_get_size(eeeapplet));
  gtk_widget_show (tmpimage);
  gtk_container_add(GTK_CONTAINER(eeeapplet->scriptbutton),tmpimage);
  
  eeeapplet->scriptmenu=menu=gtk_menu_new();
  gtk_menu_attach_to_widget(eeeapplet->scriptmenu,eeeapplet->scriptbutton,NULL);
  
  /* Populate the script menu */
  
   DIR *dp;
   struct dirent *ep;
   DIR *subdp;
   struct dirent *subep;
   char fullpath[MAXNAMELENGTH];
   char rootpath[MAXNAMELENGTH];
   char subfullpath[MAXNAMELENGTH];
   GtkWidget *image;
   GtkWidget *rootmenu;
   GtkWidget *submenu;
   GtkWidget *subfather;
   int added=0;
       
   sprintf(rootpath,"%s%s",getenv("HOME"),SCRIPTSDIR);
   dp = opendir (rootpath);
     
   if (dp != NULL)
	 {
	   while (ep = readdir (dp)) {
		  sprintf(fullpath,"%s%s",rootpath,ep->d_name);
		  if ((strncmp(ep->d_name,".",1)!=0)&&(isDir(fullpath))) {
			  added=1;
			  
			  image=gtk_image_new_from_icon_name(ICON_SCRIPT,GTK_ICON_SIZE_MENU);
			  rootmenu=gtk_image_menu_item_new_with_label(ep->d_name); 
			  gtk_image_menu_item_set_image(rootmenu,image);
			  gtk_menu_append(eeeapplet->scriptmenu,rootmenu);
			  gtk_widget_show(rootmenu);
			  
			   subfather = gtk_menu_new ();

			   subdp = opendir (fullpath);
				 
			   if (subdp != NULL)
				 {
				   while (subep = readdir (subdp)) {
					  sprintf(subfullpath,"%s/%s",fullpath,subep->d_name);
					  if ((strncmp(subep->d_name,".",1)!=0)&&(!isDir(subfullpath))) {
				  submenu=gtk_menu_item_new_with_label(subep->d_name);
				  gtk_menu_shell_append(subfather,submenu);
				  gtk_widget_show(submenu);
	    g_signal_connect_swapped (G_OBJECT (submenu), "activate",
		                      G_CALLBACK (script_run), 
                                      (gpointer) g_strdup (subfullpath));
					}
				 }
				   (void) closedir (subdp);
			   }
			  gtk_menu_item_set_submenu(rootmenu,subfather);

			  
           }
         }
	   (void) closedir (dp);
	} 
	
	if (!added) {
			sprintf(fullpath,"Please add executables into ~%s<subdir>/<scriptname>",SCRIPTSDIR);
			  image=gtk_image_new_from_icon_name(ICON_SCRIPT,GTK_ICON_SIZE_MENU);
			  rootmenu=gtk_image_menu_item_new_with_label(fullpath); 
			  gtk_image_menu_item_set_image(rootmenu,image);
			  gtk_menu_append(eeeapplet->scriptmenu,rootmenu);
			  gtk_widget_show(rootmenu);
		
	}

  /* battery charge */
  eeeapplet->gauge = gtk_progress_bar_new();
  
  gtk_widget_set_size_request(GTK_WIDGET( eeeapplet->gauge ),
				    10,xfce_panel_plugin_get_size(eeeapplet));
  gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(eeeapplet->gauge),
					 GTK_PROGRESS_BOTTOM_TO_TOP);
  gtk_widget_show(eeeapplet->gauge);
  gtk_container_add(GTK_CONTAINER(eeeapplet->hvbox),eeeapplet->gauge);

  /* cpu speed */
  
  eeeapplet->label=gtk_label_new(_(""));
  gtk_widget_set_size_request(GTK_WIDGET( eeeapplet->label ),
				    70,xfce_panel_plugin_get_size(eeeapplet));  
  gtk_widget_show(eeeapplet->label);
  gtk_container_add(GTK_CONTAINER(eeeapplet->hvbox),eeeapplet->label);

  /* signals */
  g_signal_connect (G_OBJECT (eeeapplet->button), "clicked",
                    G_CALLBACK (popup_networks), eeeapplet);
  g_signal_connect (G_OBJECT (eeeapplet->scriptbutton), "clicked",
                    G_CALLBACK (popup_scripts), eeeapplet);
     
  g_timeout_add(1000,G_CALLBACK(tick),eeeapplet);
  return eeeapplet;
}



static void
eeeapplet_free (XfcePanelPlugin *plugin,
             EEEAppletPlugin    *eeeapplet)
{
  GtkWidget *dialog;

  /* check if the dialog is still open. if so, destroy it */
  dialog = g_object_get_data (G_OBJECT (plugin), "dialog");
  if (G_UNLIKELY (dialog != NULL))
    gtk_widget_destroy (dialog);

  /* destroy the panel widgets */
  gtk_widget_destroy (eeeapplet->hvbox);
  
  /* cleanup the settings */
  if (G_LIKELY (eeeapplet->setting1 != NULL))
    g_free (eeeapplet->setting1);

  /* free the plugin structure */
  panel_slice_free (EEEAppletPlugin, eeeapplet);
}



static void
eeeapplet_orientation_changed (XfcePanelPlugin *plugin,
                            GtkOrientation   orientation,
                            EEEAppletPlugin    *eeeapplet)
{
  /* change the orienation of the box */
  xfce_hvbox_set_orientation (XFCE_HVBOX (eeeapplet->hvbox), orientation);
}



static gboolean
eeeapplet_size_changed (XfcePanelPlugin *plugin,
                     gint             size,
                     EEEAppletPlugin    *eeeapplet)
{
  GtkOrientation orientation;

  /* get the orientation of the plugin */
  orientation = xfce_panel_plugin_get_orientation (plugin);

  /* set the widget size */
  if (orientation == GTK_ORIENTATION_HORIZONTAL)
    gtk_widget_set_size_request (GTK_WIDGET (plugin), -1, size);
  else
    gtk_widget_set_size_request (GTK_WIDGET (plugin), size, -1);

  /* we handled the orientation */
  return TRUE;
}



static void
eeeapplet_construct (XfcePanelPlugin *plugin)
{
  EEEAppletPlugin *eeeapplet;
  int x;


  if (!notify_is_initted())
  	notify_init("EEEApplet");
  	
  for (x=0;x<MAXNETWORKS;x++) strcpy(connected[x],"");
  timetobattery=0;
  timetocpu=0;
  batterymax=-1;
  batterylow=0;
  
  /* setup transation domain */
  xfce_textdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");

  /* create the plugin */
  eeeapplet = eeeapplet_new (plugin);

  /* add the ebox to the panel */
  gtk_container_add (GTK_CONTAINER (plugin), eeeapplet->ebox);

  /* show the panel's right-click menu on this ebox */
  xfce_panel_plugin_add_action_widget (plugin, eeeapplet->button);

  /* connect plugin signals */
  g_signal_connect (G_OBJECT (plugin), "free-data",
                    G_CALLBACK (eeeapplet_free), eeeapplet);

  g_signal_connect (G_OBJECT (plugin), "save",
                    G_CALLBACK (eeeapplet_save), eeeapplet);

  g_signal_connect (G_OBJECT (plugin), "size-changed",
                    G_CALLBACK (eeeapplet_size_changed), eeeapplet);

  g_signal_connect (G_OBJECT (plugin), "orientation-changed",
                    G_CALLBACK (eeeapplet_orientation_changed), eeeapplet);

  /* show the configure menu item and connect signal */
  // xfce_panel_plugin_menu_show_configure (plugin);
  /*
  g_signal_connect (G_OBJECT (plugin), "configure-plugin",
                    G_CALLBACK (eeeapplet_configure), eeeapplet);
*/
  /* show the about menu item and connect signal */
  xfce_panel_plugin_menu_show_about (plugin);
  g_signal_connect (G_OBJECT (plugin), "about",
                    G_CALLBACK (eeeapplet_about), NULL);
}
